#!/usr/bin/env python
# vim: set ts=4 sw=4 sts=0 :
# ScoreXML.py :
#
# Copyright (C) 2007, Shu KONNO <owa@bg.wakwak.com>
# This program distributed under GPL (GNU General Public License).
from xml.parsers.expat import ParserCreate
import StringIO
import string
import gettext
import os
__version__ = '0.5'

class SimpleXML(object):
	def __init__(self, tags, path=None):
		#print 'SimpleXML:', path
		self.__tags = tags
		self.__path = path
		self.clear()

	def clear(self):
		self.__tag_stack = []
		self.__info = {}
		for tag in self.__tags:
			self.__info[tag] = {}

	def setAttr(self, tag, attr, s):
		if self.__info.has_key(tag):
			items = self.__info[tag]
			items[attr] = s

	def getAttr(self, tag, attr):
		if self.__info.has_key(tag):
			return self.__info[tag].get(attr, '')

	def setData(self, tag, s):
		if self.__info.has_key(tag):
			items = self.__info[tag]
			items['data'] = s

	def getData(self, tag):
		if self.__info.has_key(tag):
			return self.__info[tag].get('data', '')

	def __start_element(self, name, attr):
		#print '__start_element:', name, attr
		if name in self.__tags:
			self.__info[name] = attr
		self.__tag_stack.append(name)
		#print '__start_element:', self.__tag_stack

	def __end_element(self, name):
		#print '__end_element:', name
		self.__tag_stack.pop()

	def __char_data(self, data):
		name = self.__tag_stack[-1]
		#data = string.strip(data, string.whitespace)
		data = data.strip()
		if data:
			data = unicode(data)
			#print '__char_data:', name, type(data), data
			d = self.__info
			if d.has_key(name):
				s = d[name].get('data', '')
				d[name]['data'] = s + data

	def construct(self, f):
		self.clear()
		p = ParserCreate()
		p.StartElementHandler = self.__start_element
		p.EndElementHandler = self.__end_element
		p.CharacterDataHandler = self.__char_data
		try:
			p.ParseFile(f)
		except Exception, e:
			print 'SimpleXML::error:', e

	def serialize(self):
		buf = u'<?xml version="1.0" encoding="utf-8" ?>\n'
		buf += '<ZShogi version="%s">\n' % __version__
		for k in self.__tags:
			if self.__info.has_key(k):
				d = self.__info[k]
				buf += '\t<%s' % k
				for a, v in d.items():
					if a != 'data':
						buf += ' %s="%s"' % (a, v)
				if d.has_key('data'):
					buf += '>'
					buf += d['data']
					buf += '</%s>\n' % k
				else:
					buf += ' />\n'
		buf += '</ZShogi>\n'
		return buf

	def load(self, path=None):
		if path is None:
			path = self.__path
		f = open(path, 'r')
		s = f.read()
		f.close()
		ram = StringIO.StringIO(s)
		self.construct(ram)

	def save(self, path=None):
		if not path:
			path = self.__path
		u = self.serialize()
		f = open(path, 'w')
		f.write(u.encode('utf-8'))
		f.close()

	def _getInfo(self):
		return self.__info


_tag_URI = 'uri'
_tag_FilePath = 'filepath'
_tag_Title = 'title'
_tag_Comment = 'comment'
_tag_Author = 'author'
_tag_White = 'white'
_tag_Black = 'black'
_tag_Diagram = 'diagram'
_tag_Moving = 'moving'
_attr_GameType = 'type'
_attr_StartSide = 'side'
_attr_Cursor = 'cursor'

class ScoreXML(object):
	tags = (_tag_URI, _tag_FilePath, _tag_Title, _tag_Comment,
		_tag_Author, _tag_White, _tag_Black, _tag_Diagram, _tag_Moving)
	def __init__(self, path=None, encode='utf-8'):
		#print 'ScoreXML:', path
		self.__xml = SimpleXML(ScoreXML.tags, path)
		self.__encode = encode

	def clear(self):
		self.__xml.clear()

	def construct(self, s):
		ram = StringIO.StringIO(s)
		self.__xml.construct(ram)

	def serialize(self):
		return self.__xml.serialize()

	def savefile(self, path=None):
		self.__xml.save(path)

	def loadfile(self, path=None):
		self.__xml.load(path)

	def getScore(self):
		d = self.getDiagram()
		if not d:
			d = self.getGameType()
		s = self.getMoving()
		return d + '/' + s

	def __setName(self, tag, s):
		if isinstance(s, unicode):
			print 'error:', type(s), s
		self.__xml.setName(tag, s)

	def __setAttr(self, tag, attr, s):
		if isinstance(s, unicode):
			print 'error:', type(s), s
		self.__xml.setAttr(tag, attr, s)

	def __setData(self, tag, s):
		if not isinstance(s, unicode):
			s = unicode(s, self.__encode)
		s = s.strip()
		#print 'setData: %s (%s)' % (tag, s)
		self.__xml.setData(tag, s)

	def setURI(self, s): self.__setData(_tag_URI, s)
	def setFilePath(self, s): self.__setData(_tag_FilePath, s)
	def setTitle(self, s): self.__setData(_tag_Title, s)
	def setComment(self, s): self.__setData(_tag_Comment, s)
	def setAuthor(self, s): self.__setData(_tag_Author, s)
	def setWhite(self, s): self.__setData(_tag_White, s)
	def setBlack(self, s): self.__setData(_tag_Black, s)
	def setMoving(self, s): self.__setData(_tag_Moving, s)
	def setDiagram(self, s): self.__setData(_tag_Diagram, s)
	def setGameType(self, s): self.__setAttr(_tag_Diagram, _attr_GameType, s)
	def setStartSide(self, s): self.__setAttr(_tag_Diagram, _attr_StartSide, str(s))
	def setCursor(self, s): self.__setAttr(_tag_Moving, _attr_Cursor, str(s))

	def getURI(self): return self.__xml.getData(_tag_URI)
	def getFilePath(self): return self.__xml.getData(_tag_FilePath)
	def getTitle(self): return self.__xml.getData(_tag_Title)
	def getComment(self): return self.__xml.getData(_tag_Comment)
	def getAuthor(self): return self.__xml.getData(_tag_Author)
	def getWhite(self): return self.__xml.getData(_tag_White)
	def getBlack(self): return self.__xml.getData(_tag_Black)
	def getMoving(self): return self.__xml.getData(_tag_Moving)
	def getDiagram(self): return self.__xml.getData(_tag_Diagram)
	def getGameType(self): return self.__xml.getAttr(_tag_Diagram, _attr_GameType)
	def getStartSide(self): return self.__xml.getAttr(_tag_Diagram, _attr_StartSide)
	def getCursor(self): return self.__xml.getAttr(_tag_Moving, _attr_Cursor)

	def dump(self):
		print '--- dump ---'
		info = self.__xml._getInfo()
		for tag in ScoreXML.tags:
			print '<%s>' % tag,
			if info.has_key(tag):
				d = info[tag]
				for k, v in d.items():
					print ' %s:%s' % (k, v),
			print
		#print '--- raw ---'
		#print info


if __name__ == '__main__':
	gettext.install('zbrowser', '../i18n', unicode=True)
	path = 'score.xml'
	f = ScoreXML(path)
	if 1:
		f.clear()
		f.setURI(' http://localhost:9080/zope/test/ZShogi/CH01 2\n')
		f.setTitle(_('%TestTitle%'))
		f.setComment(_('%TestComment%'))
		f.setAuthor(_('%TestAuthor%'))
		f.setWhite(_('%TestWhite%'))
		f.setBlack(_('%TestBlack%'))
		f.setGameType('H0')
		f.setStartSide('1')
		f.setCursor('2')
		f.setMoving('76773433262784835657')
		f.dump()
		f.savefile()

	f.loadfile()
	f.dump()

